PERMISSIONS_TREE = {
    'root': ['admin'],
    'admin': ['execution'],
    'execution': []
}

PROTOCOLS = [
]

PLUGINS = [
    'bot',
    'hello',
]

PERMS_METHODS = [
    'nekbot.core.permissions.perms_from_settings',
]

SYMBOL = '!'